from datamana.ipc import Semaphore, MQueue, SharedMemory

class Base():
    def __init__(self, name):
        self.data_name = f'datamana{name}'
        self.data_meta_name = f'{self.data_name}meta'
        self.data_share_name = f'{self.data_name}share'
        self.data_sem_name = f'{self.data_name}sem'
        self.data_mq_tx_name =f'{self.data_name}mqtx'
        self.data_mq_rx_name =f'{self.data_name}mqrx'
        self.sem = Semaphore(self.data_sem_name)
        self.mq_tx = MQueue(self.data_mq_tx_name)
        self.mq_rx = MQueue(self.data_mq_rx_name)
        self.name2shm = {}

    def write_shared_data(self, shm_name, data, size):
        if shm_name in self.name2shm:
            shm = self.name2shm[shm_name]
        else:
            shm = SharedMemory(name=shm_name, size=size)
            self.name2shm[shm_name] = shm
        shm.resize(size)

        shm.buf[:size] = data[:]

    def server_send(self, msg, msg_prio=0):
        return self.mq_tx.send(msg, msg_prio)

    def server_recv(self):
        return self.mq_rx.receive()

    def client_send(self, msg, msg_prio=0):
        return self.mq_rx.send(msg, msg_prio)

    def client_recv(self):
        return self.mq_tx.receive()



import os, mmap
from datamana._C import (
    Semaphore as __Semaphore,
    MQueue as __MQueue,
    SharedMemory as __SharedMemory,
)

class Semaphore(__Semaphore):
    def __init__(self, name, oflag=os.O_CREAT | os.O_RDWR, mode=0o600, value=1):
        super().__init__()
        errno = self.open(name, oflag, mode, value)
        if errno != 0:
            raise RuntimeError(f'create Semaphore failed! <{os.strerror(errno)}>')

del __Semaphore

class MQueue(__MQueue):
    def __init__(self, name, oflag=os.O_CREAT | os.O_RDWR, mode=0o600, msgsize=1, maxmsg=1):
        super().__init__()
        self.msgsize = msgsize
        self.maxmsg = maxmsg
        name = '/' + name
        errno = self.open(name, oflag, mode)
        if errno != 0:
            raise RuntimeError(f'create MQueue failed! <{os.strerror(errno)}>')

del __MQueue

class SharedMemory(__SharedMemory):
    def __init__(self, name, size, oflag=os.O_CREAT | os.O_RDWR, mode=0o600):
        super().__init__()
        name = '/' + name
        errno = self.open(name, oflag, mode)
        if errno != 0:
            raise RuntimeError(f'create SharedMemory failed! <{os.strerror(errno)}>')
        self.resize(size)

    def resize(self, size, strict=False):
        shm_size = self.size
        if shm_size < size or (shm_size > size and strict):
            os.ftruncate(self.fd, size)
            shm_size = size
            self._mmap = mmap.mmap(self.fd, size)
            self._buf = memoryview(self._mmap)

    @property
    def size(self):
        stats = os.fstat(self.fd)
        shm_size = stats.st_size
        return shm_size

    @property
    def buf(self):
        return self._buf

del __SharedMemory
